/**
 * Define api of apeman-api-publish
 * @function define
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging=false] - Enable logger
 * @param {string} [options.publishDir='public'] - Directory names to publish
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')
const spec = require('./spec')
const apemanApiPath = require('apeman-api-path')
const apemanApiJSON = require('apeman-api-json')
const apemanApiMap = require('apeman-api-map')
const apemanApiScss = require('apeman-api-scss')
const apemanApiBrowserify = require('apeman-api-browserify')
const apemanApiCC = require('apeman-api-closurecompiler')
const load = require('apeman-api-client/load')

/** @lends define */
function define (options = {}) {
  let {
    logging = true,
    root = process.cwd(),
    publishDir = 'public',
    transformModules = {
      json: require('abrowserify/transforms/json_transform')
    }
  } = options

  let logger = aslogger({ disabled: !logging })

  let pathApi = apemanApiPath({ logging, root })
  let jsonApi = apemanApiJSON({ logging })
  let mapApi = apemanApiMap({ logging })
  let scssApi = apemanApiScss({ logging })
  let browserifyApi = apemanApiBrowserify({
    logging,
    transformModules
  })
  let ccApi = apemanApiCC({ logging })

  return {
    /**
     * Test the reachability of the api.
     * @method ping
     * @param {string} [pong='pong'] - Pong message to return
     * @returns {string} - Pong message
     */
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    /**
     * Compile scss files to publish
     * @method scss
     * @param {string} srcDir - Source root directory
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    scss (srcDir, options = {}) {
      let {
        pattern = '*.scss',
        destDir = 'css'
      } = options
      const s = this
      return co(function * () {
        let scss = yield load(scssApi, { $$ctx: s })
        return yield scss.compileAll(pattern, {
          cwd: srcDir,
          out: `${publishDir}/${destDir}`
        })
      })
    },

    /**
     * Bundle javascript file
     * @method bundle
     * @param {string} srcDir - Source directory name
     * @param {Object} [options={}] - Optional settings
     * @param {string[]} [options.externals] - External modules
     * @returns {Promise}
     */
    bundle (srcDir, options = {}) {
      let {
        pattern = '*.js',
        destDir = 'js',
        transforms = [],
        externals = []
      } = options
      const s = this
      return co(function * () {
        let path = yield load(pathApi, { $$ctx: s })
        let browserify = yield load(browserifyApi, { $$ctx: s })
        let out = `${publishDir}/${destDir}`
        let bundled = yield browserify.bundleAll(pattern, {
          cwd: srcDir,
          out,
          transforms,
          externals
        })
        return Promise.all(
          bundled.map((bundled) => path.relative(out, bundled))
        )
      })
    },

    /**
     * Bundle external javascripts
     * @method bundleExternals
     * @param {string} filename - File name to bundle to
     * @param {string[]} externals - External module names
     * @param {Object} [options={}] - Optional settings
     * @returns {Promise}
     */
    bundleExternals (filename, externals = [], options = {}) {
      let {
        destDir = 'js'
      } = options
      const s = this
      return co(function * () {
        let browserify = yield load(browserifyApi, { $$ctx: s })
        let dest = `${publishDir}/${destDir}/${filename}`
        return yield browserify.bundle(null, dest, { require: externals })
      })
    },

    /**
     * Create manifest json
     * @method manifest
     * @param {string} src - Source file
     * @returns {*}
     */
    manifest (src) {
      const s = this
      return co(function * () {
        let path = yield load(pathApi, { $$ctx: s })
        let json = yield load(jsonApi, { $$ctx: s })
        src = yield path.absolute(src)
        return yield json.convert(src, `${publishDir}/manifest.json`)
      })
    },

    /**
     * @method extractMaps
     * @param {string} pattern - Glob filename patterns
     * @param {Object} options - Optional settings
     * @returns {Promise}
     */
    extractMaps (pattern, options = {}) {
      let { ignore = [] } = options
      const s = this
      return co(function * () {
        let map = yield load(mapApi, { $$ctx: s })
        return yield map.mapAll(pattern, {
          ignore,
          cwd: `${publishDir}/js`
        })
      })
    },

    /**
     * Compile with closure compiler
     * @method closureCompile
     * @param {string} pattern - Glob filename patterns
     * @param {Object} options - Optional settings
     * @returns {Promise}
     */
    closureCompile (pattern, options = {}) {
      const s = this
      let {
        ignore = [ '*.cc.js' ],
        ext = '.cc.js'
      } = options
      return co(function * () {
        let cc = yield load(ccApi, { $$ctx: s })
        return yield cc.compileAll(pattern, {
          cwd: `${publishDir}/js`,
          out: `${publishDir}/js`,
          ignore,
          ext
        })
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: spec
  }
}

module.exports = define
